from mitmproxy.certs import DEFAULT_EXP

ALL = '*'

CADIR = '~/.padsniff'
DEFAULT_CN = 'Puzzle & Dragons HTTP Sniffer'
DEFAULT_ORG = 'padsniff'

GUNGHO_API_ENDPOINT = '/api.php'
GUNGHO_USER_AGENT = 'GunghoPuzzleAndDungeon'
